unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls;

type

  // define the Interface
  ITestInterface =  interface
      function OnlyDummy: integer;
    end;

  { TIntfComp }

  TIntfComp = class(TComponent)
  private
    FObjectHasInterface: ITestInterface;
    function GetObjectHasInterface: ITestInterface;
    procedure SetObjectHasInterface(AValue: ITestInterface);
  protected

  public

  published
    property ObjectHasInterface: ITestInterface read GetObjectHasInterface write SetObjectHasInterface;
  end;

  { TMyTestObject }

  TMyTestObject = Class(TObject)
  private
    FIntf: IInterface;
  Public
    Constructor Create;
    Destructor Destroy;override;
  Published
    Property IntfField: IInterface read FIntf write FIntf;
end;

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    procedure Button1Click(Sender: TObject);
  private

  public

  end;

var
  Form1: TForm1;

implementation

uses
  Typinfo;

{$R *.lfm}

{ TIntfComp }

function TIntfComp.GetObjectHasInterface: ITestInterface;
begin
  Result:= FObjectHasInterface;
end;

procedure TIntfComp.SetObjectHasInterface(AValue: ITestInterface);
begin
  FObjectHasInterface:= AValue;
end;

{ TMyTestObject }

constructor TMyTestObject.Create;
begin
//  FIntf:=TInterfacedObject.Create;
end;

destructor TMyTestObject.Destroy;
begin
  inherited Destroy;
end;


Const TypeNames : Array [TTypeKind] of string[15] =
                    ('Unknown','Integer','Char','Enumeration',
                     'Float','Set','Method','ShortString','LongString',
                     'AnsiString','WideString','Variant','Array','Record',
                     'Interface','Class','Object','WideChar','Bool','Int64','QWord',
                     'DynamicArray','RawInterface','ProcVar','UnicodeString','UnicodeChar',
                     'Helper','File','ClassRef','Pointer');

Const OrdinalTypes = [tkInteger,tkChar,tkENumeration,tkbool];


Function ProcType (PP : Byte) : String;

begin
  Case PP and 3 of
    ptfield   : Result:='from Field';
    ptstatic  : Result:='with static method';
    ptVirtual : Result:='with virtual method';
    ptconst   : Result:='with Const';
   end;
end;


//Procedure DumpTypeInfo (O : TMyTestObject; MLines: TStrings);
Procedure DumpTypeInfo (O : TObject; MLines: TStrings);

Var
    PT : PTypeData;
    PI : PTypeInfo;
    I  : Longint;
    PP : PPropList;

begin
  PI:=O.ClassInfo;
  MLines.Add('Type kind : '+ TypeNames[PI^.Kind]);
  MLines.Add('Type name : '+PI^.Name);
  PT:=GetTypeData(PI);
  //DumpMem(PByte(PI));
  If PT^.ParentInfo=Nil then
    MLines.Add('Object has no parent info')
  else
    MLines.Add('Object has parent info');
  MLines.Add('Property Count : '+IntToStr(PT^.PropCount));
  MLines.Add('Unit name      : '+PT^.UnitName);
  GetMem (PP,PT^.PropCount*SizeOf(Pointer));
  GetPropInfos(PI,PP);
  For I:=0 to PT^.PropCount-1 do
  If PP^[i]<>Nil then
    With PP^[I]^ do
      begin
      MLines.Add('Property name : '+Name);
      MLines.Add(' Type kind: '+TypeNames[PropType^.Kind]);
      MLines.Add(' Type Name: '+PropType^.Name);
      If GetProc=Nil then Write ('No');
      MLines.Add(' Getproc available');
      If SetProc=Nil then Write ('No');
      MLines.Add(' Setproc available');
      If StoredProc=Nil then MLines.Add('No');
      MLines.Add(' Storedproc available');
      MLines.Add(' Get property '+proctype(Propprocs));
      MLines.Add(' Set Property '+proctype(propprocs shr 2));
      MLines.Add(' Stored Property '+proctype(propprocs shr 4));
      MLines.Add(' Default : '+Default.ToString+' Index : '+Index.ToString);
      MLines.Add(' NameIndex : '+NameIndex.ToString);
      end;
    FreeMem (PP);
end;

//Procedure TestGet (O : TMyTestObject; MLines: TStrings);
Procedure TestGet (O : TObject; MLines: TStrings);

Var
    PT : PTypeData;
    PI : PTypeInfo;
    I,J : Longint;
    PP : PPropList;
    prI : PPropInfo;
    Intf : IInterface;
    HStr : String;
begin
  PI:=O.ClassInfo;
  MLines.Add( 'Type kind : '+TypeNames[PI^.Kind]);
  MLines.Add( 'Type name : '+PI^.Name);
  PT:=GetTypeData(PI);
  If PT^.ParentInfo=Nil then
    MLines.Add( 'Object has no parent info')
  else
    MLines.Add( 'Object has parent info');
  MLines.Add( 'Property Count : '+(PT^.PropCount).ToString);
  MLines.Add( 'Unit name      : '+PT^.UnitName);
  GetMem (PP,PT^.PropCount*SizeOf(Pointer));
  GetPropInfos(PI,PP);
  For I:=0 to PT^.PropCount-1 do
    begin
    pri:=PP^[i];
    With Pri^ do
      begin
      Hstr:= '(Examining ' +name+' : Type : '+TypeNames[PropType^.Kind]+', ';
      If (Proptype^.kind in Ordinaltypes) Then
        begin
        J:=GetOrdProp(O,pri);
        HStr:= Hstr+ 'Value : ' + j.ToString;
        If PropType^.Kind=tkenumeration then
          HStr:= Hstr+ '(='+GetEnumName(Proptype,J)+')'
        end
      else
        Case pri^.proptype^.kind of
          tkfloat :  begin
                     HStr:= Hstr+ 'Value : ' + GetFloatProp(O,pri).ToString;
                     end;
        tkAstring : begin
                    HStr:= Hstr+ 'value : ' + GetStrProp(O,Pri);
                    end;
        tkInterface : begin
                       //HStr:= Hstr+ 'value : ' +  PtrUInt(GetObjectProp(O,Pri)).ToString;
                       HStr:= Hstr+ 'value : ' +  PtrUInt(GetInterfaceProp(O,Pri)).ToString;
                       { play a little bit with the interface to test SetInterfaceProp }
                       SetInterfaceProp(O,Pri,TInterfacedObject.Create);
                     end;
        tkClass   : begin
                       HStr:= Hstr+ 'value : '+ PtrUInt(GetObjectProp(O,Pri)).ToString;
                     end;
        else
          HStr:= Hstr+ 'Untested type:' + ord(pri^.proptype^.kind).ToString;
        end;
          MLines.Add(Hstr + ')');
      end;
    end;
  FreeMem (PP);
end;


{ TForm1 }

procedure TForm1.Button1Click(Sender: TObject);
Var
  Obj : TMyTestObject;
  IObj : TIntfComp;
begin
  Memo1.Clear;
  Memo1.Append('----- Test Obj start -----');
  Obj:=TMyTestObject.Create;
  Memo1.Append('*** DumpTypeInfo ***');
  DumpTypeInfo(Obj,Memo1.Lines);
//  PrintObject(Obj);
  Memo1.Append('*** TestGet ***');
  testget(Obj,Memo1.Lines);
  Obj.Free;
  Memo1.Append('----- Test IObj start -----');
  IObj:= TIntfComp.Create(nil);
  Memo1.Append('*** DumpTypeInfo ***');
  DumpTypeInfo(IObj,Memo1.Lines);
//  PrintObject(Obj);
  Memo1.Append('*** TestGet ***');
  testget(IObj,Memo1.Lines);
  IObj.Free;
  Memo1.Append('----- Test end -----');
end;



end.

